/*
 * Copyright (C) 2006-2007 Funambol
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package com.funambol.mail;

import java.io.DataInputStream;
import java.io.DataOutputStream;
import java.io.IOException;

import com.funambol.storage.Serializable;
import com.funambol.storage.ComplexSerializer;

/**
 * A BodyPart object is foreseen to be inserted into a Multipart container,
 * itself inserted in a multi-part Message with MIME Content-Type equal to
 * "multipart"
 */
public class BodyPart extends Part implements Serializable {

    // -------------------------------------------------------------- Constants

    /**
     * The name of the MIME Content-Disposition header attribute
     */
    public static final String CONTENT_DISPOSITION = "Content-Disposition";

    /**
     * A possible value for the the "Content-Disposition" header field
     */
    public static final String CD_ATTACHMENT = "attachment";

    /**
     * A possible value for the the "Content-Disposition" header field
     */
    public static final String CD_INLINE = "inline";

    private static final String CD_FILENAME = "filename";

    // ------------------------------------------------------------- Attributes

    /**
     * A Multipart can contain one or more BodyParts, and each BodyPart has a
     * reference to its container, if present
     */
    private Multipart container;

    /** The content disposition (inline/attachment) */
    private String disposition;

    /** The filename for this bodypart, if specified */
    private String filename;

    /** The content of this bodypart */
    private Object content;


    // ----------------------------------------------------------- Constructors

    public BodyPart() {
        // super(); // init Part

        // container = null;
        // disposition = INLINE;
        // filename = null;
        // content = null;
        this(null);
    }


    public BodyPart(Multipart container) {

        super(); // init Part

        this.container = container;
        disposition = CD_INLINE;
        filename = null;
        content = null;

    }


    // --------------------------------------------------------- Public Methods

    /**
     * Returns the value of the "Content-Disposition" header field. This
     * represents the disposition of this BodyPart. The disposition describes
     * how the part should be presented to the user.
     * 
     * @return The value of the Content-Disposition header field of this part or
     *         <code>null</code> if not present.
     */
    public String getDisposition() {
        return disposition;
    }


    /**
     * Returns the value of the filename attribute of the "Content-Disposition"
     * header field. This represents the filename for this BodyPart (suggests to
     * the receiving mail client how to name it).
     * 
     * @return The value of the filename of this BodyPart, or <code>null</code>
     *         if not present.
     */
    public String getFileName() {
        return filename;
    }


    /**
     * Returns the container Multipart of this BodyPart.
     * 
     * @return A reference to the Multipart object this part is contained in, if
     *         the message is a multi-part message
     */
    public Multipart getContainer() {
        return container;
    }


    /**
     * Returns the content of of this BodyPart.
     * 
     * @return A reference to the Object in this BodyPart, or null if empty.
     */
    public Object getContent() {
        return content;
    }


    /**
     * 
     * @return The textual content of this BodyPart, if it is a String, or null
     *         otherwise.
     */
    public String getTextContent() {
        if (content instanceof String) {
            return (String)content;
        } else {
            return null;
        }
    }


    /**
     * Sets the container Multipart of this BodyPart.
     * 
     * @param container
     *            A reference to the Multipart object to set as container
     */
    public void setContainer(Multipart container) {
        this.container = container;
    }


    /**
     * Sets the content of this BodyPart. It can be a Multipart or text (a
     * String) or a byte array (e.g. a jpeg attachment). 
     * 
     */
    public void setContent(Object content) throws MailException {
        if (content instanceof Multipart) {
            ((Multipart)content).setContainer(this);
            contentType = "multipart/mixed";
        } else if (content instanceof String) {
            contentType = "text/plain";
        } else if (content instanceof Message) {
            contentType = "application/eml"; 
        } else if (content instanceof byte[]) {
            contentType = "application/octet-stream";
        } else {
            throw (new MailException(
                            MailException.INVALID_CONTENT,
                            "Invalid content: " + content.getClass().getName()));
        }

        this.content = content;
    }


    /**
     * Sets the content of this BodyPart. This can be another Multipart (for
     * instance, a multipart/mixed message can contains a multipart/alternative
     * in one bodypart), String (in case of a text content) or a byte array for
     * binary data.
     * 
     * @param content
     *            The content object
     * @param type
     *            The MIME type of the content
     */
    public void setContent(Object content, String type) throws MailException {
        setContentType(type);
        setContent(content);
    }


    /**
     * Set the content disposition of this BodyPart
     * 
     * @param disp
     *            One of ATTACHMENT or INLINE
     */
    public void setDisposition(String disp) {
        disposition = disp;
    }


    /**
     * Sets the file name of this BodyPart
     * 
     * @param fn
     *            The file name of the attachment
     */
    public void setFileName(String fn) {
        filename = fn;
    }


    // ---------------------------------- interface Serializable implementation

    public void serialize(DataOutputStream dout) throws IOException {
        
        // Write Part attributes
        dout.writeUTF(contentType);
        ComplexSerializer.serializeHashTable(dout, headers);
        
        if (disposition != null) {
            dout.writeBoolean(true);
            dout.writeUTF(disposition);
        } else {
            dout.writeBoolean(false);
        }
            
        if (filename != null) {
            dout.writeBoolean(true);
            dout.writeUTF(filename);
        } else {
            dout.writeBoolean(false);
        }
            
        /*
         * the container is not stored: the container will call setContainer on
         * deserialize.
         */
         ComplexSerializer.serializeObject(dout, content);
    }


    public void deserialize(DataInputStream din) throws IOException {

        contentType = din.readUTF();
        headers = ComplexSerializer.deserializeHashTable(din);
        
        if (din.readBoolean())
            disposition = din.readUTF();
        if (din.readBoolean())
            filename = din.readUTF();
        
        content = ComplexSerializer.deserializeObject(din);

        // Set the container reference for Multipart
        if (content instanceof Multipart) {
            ((Multipart)content).setContainer(this);
        }

    }

}
